package neuronespack;

/**
 * Package Neurones
 * Fonctionalits de base des rseaux neuronaux.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */


import java.io.*;
import java.util.*;

/**
 * Gestion des donnes d'apprentissage et de test.
 * Pour les fichiers, la premire ligne contient :
 * type (apprend/test); nombre de vecteurs d'entre; taille de ces vecteurs;
 * nombre de vecteurs de sortie; taille de ces vecteurs
 * Les lignes suivantes contiennent :
 * les scalaires des vecteurs d'entre suivis des sorties dsires (si apprend)
 */

/**
 * <p>Title: Donnes d'entre</p>
 * <p>Description: Gestion des donnes d'entre.</p>
 */
public class DonneesEntree implements GroupeUnites {
  public static final char SEPARATEUR = ';';

  /** taille des vecteurs d'entre */
  private int tailleVecteurEntree;
  /** Vecteur d'entre courante */
  private double vecteurEntree[];
  /** taille des vecteurs de sortie */
  private int tailleVecteurSortie;
  /** Vecteur sortie dsir */
  private double vecteurSortieDesire[];
  /** Liste de pointeurs sur les units */
  private Vector lesUnites;
  /** Fichier d'entre */
  protected FileReader fichierEntrees;
  /** Position courante dans le fichier */
  private int posCrt;

  /**
   * Constructeur : Cre un objet de gestion des entres
   * @param tve int Taille vecteur entre
   * @param tvs int Taille vecteur sortie
   */
  public DonneesEntree(int tve, int tvs) {
    posCrt = 0;
    tailleVecteurEntree = tve;
    tailleVecteurSortie = tvs;
    vecteurEntree = new double[tailleVecteurEntree];
    lesUnites = new Vector(tailleVecteurEntree);
    for (int i=0; i <tailleVecteurEntree; i++) {
      lesUnites.addElement(new UniteExterne(i));
    }
    vecteurSortieDesire = new double[tailleVecteurSortie];
  }

  /** Mise  jour du vecteur d'entre courant */
  public void setVecteurEntree(double e[]) {
    for(int i=0; i<tailleVecteurEntree; i++) {
      vecteurEntree[i] = e[i];
      ((UniteExterne) lesUnites.elementAt(i)).setSignalCourant(vecteurEntree[i]);
    }
  }

  /** Getter du vecteur d'entre */
  public double[] getVecteurEntree() {
    return vecteurEntree;
  }

  /** Getter d'une valeur de sortie */
  public double getValeurEntree(int i) {
    return vecteurEntree[i];
  }

  /** Getter taille vecteur d'entre */
  public int getTailleVecteurEntree() {
    return tailleVecteurEntree;
  }

  /** Connecte un fichier d'entre */
  public void setFichierEntree(String nomFic) {
    if(fichierEntrees != null) {
      fermeFichierDonnees();
    }
    try {
      fichierEntrees = new FileReader(nomFic);
    }  catch (IOException e) {
      System.out.println("Erreur fichier : "+e.toString()); }
  }

  /** Mise  jour du vecteur de sortie dsir courant */
  public void setVecteurSortieDesire(double e[]) {
    for(int i=0; i<tailleVecteurSortie; i++) {
      vecteurSortieDesire[i] = e[i];
    }
  }

  /** Getter du vecteur de sortie dsir */
  public double[] getVecteurSortieDesiree() {
    return vecteurSortieDesire;
  }

  /** Getter d'une valeur de sortie dsire */
  public double getValeurSortieDesiree(int i) {
    return vecteurSortieDesire[i];
  }

  /** Getter taille vecteur de sortie */
  public int getTailleVecteurSortie() {
    return tailleVecteurSortie;
  }

  /** Lit l'entree suivante du fichier */
  public void litEntreeSuivante() {
    litVecteurEntree();
  }

  /** lit l'exemple d'apprentissage suivant */
  public void litExempleSuivant() {
    litVecteurEntree();
    litVecteurSortieDesire();
  }

  /** Lecture du vecteur d'entre dans le fichier */
  private void litVecteurEntree() {
    posCrt++;
    for(int i=0; i<tailleVecteurEntree; i++) {
      vecteurEntree[i] = litVal();
      ((UniteExterne) lesUnites.elementAt(i)).setSignalCourant(vecteurEntree[i]);
    }
  }

  /** Lecture du vecteur de sortie dsir dans le fichier */
  private void litVecteurSortieDesire() {
    for(int i=0; i<tailleVecteurSortie; i++) {
      vecteurSortieDesire[i] = litVal();
    }
  }

  /** Lecture da la valeur suivante du fichier */
  private double litVal() {
    char c = '_';
    String s = "";
    try {
      c=(char)fichierEntrees.read();
      while (c!=SEPARATEUR) {
        if (c != '\n' && c != '\r') {
          s += c;
        }
        c = (char)fichierEntrees.read();
      }
    } catch (IOException e) {
        System.out.println("Erreur fichier : "+e.toString()); }
    return Double.valueOf(s).doubleValue();
  }

  /** Retourne au dbut du fichier */
  private void reInitFichier() {
    try {
      fichierEntrees.reset();
      posCrt = 0;
    } catch (IOException e) {
      System.out.println("Erreur fichier : "+e.toString()); }
  }

  /** Fermeture fichier */
  public void fermeFichierDonnees() {
    try {
      fichierEntrees.close();
    } catch (IOException e) {
      System.out.println("Erreur fichier : "+e.toString()); }
  }

  /** Getter d'une unit */
  public Unite getUnite(int u) {
    return (Unite) lesUnites.elementAt(u);
  }

  /** Getter des units */
  public Vector getLesUnites() {
    return lesUnites;
  }

  /** Getter du nombre d'units */
  public int getNbUnites() {
    return tailleVecteurEntree;
  }
}
