package somcolorapp;

/**
 * Package somcolorapp
 * Classification des couleurs avec un rseau de Kohonen.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;
import java.awt.event.*;
import baseui.*;

/**
* <p>Title: Interface utilisateur</p>
 * <p>Description: Interface utilisateur de l'applet.</p>
 */
public class SOMColorUI extends Cui { // Frame Cui
  /** Applet */
  public SOMColor applet;
  /** processus */
  public SOMColorProcess processSOM;
  /** Taille du rseau */
  public int tailleReseau;
  /** Maximum itrations */
  public int maxIterations;
  /** Nb couleurs ^1/3 */
  public int nbCouleurs;
  /** Eta0 */
  public double eta0;
  /** Sigma0 */
  public double sigma0;

  Label statusBar = new Label();
  Panel panel1 = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  Panel panel2 = new Panel();
  GridLayout gridLayout1 = new GridLayout();
  Button btnReset = new Button();
  Button btnAbout = new Button();
  Button btnGo = new Button();
  Button btnPas = new Button();
  Panel panelVue = new Panel();
  GridLayout gridLayout2 = new GridLayout();
  Panel panel3 = new Panel();
  TextField txtTaille = new TextField();
  Label label1 = new Label();
  Label label2 = new Label();
  TextField txtCouleurs = new TextField();
  Label label3 = new Label();
  TextField txtIterations = new TextField();
  GridLayout gridLayout3 = new GridLayout();
  Label label4 = new Label();
  TextField txtEta0 = new TextField();
  Label label5 = new Label();
  TextField txtSigma0 = new TextField();
  Button btnModif = new Button();

  public SOMColorUI(SOMColor ap, int t) {
    applet = ap;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    processSOM = new SOMColorProcess(this);
    process = (Cprocess) processSOM;
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setTitle(applet.comments + " " + applet.version + " " + applet.auteur);
    statusBar.setText(" Itration :");
    panel1.setLayout(borderLayout1);
    panel2.setLayout(gridLayout1);
    gridLayout1.setColumns(4);
    gridLayout1.setHgap(0);
    btnReset.setLabel("Reset");
    btnReset.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnReset_actionPerformed(e);
      }
    });

    btnAbout.setLabel("A propos");
    btnAbout.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnAbout_actionPerformed(e);
      }
    });

    btnGo.setLabel("GO");
    btnGo.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnGo_actionPerformed(e);
      }
    });
    btnPas.setLabel("Pas  Pas");
    btnPas.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnPas_actionPerformed(e);
      }
    });

    panelVue.setLayout(gridLayout2);
    panelVue.addComponentListener(new java.awt.event.ComponentAdapter() {
      public void componentResized(ComponentEvent e) {
        panelVue_componentResized(e);
      }
    });

    label1.setAlignment(Label.CENTER);
    label1.setText("Taille^1/2");
    label2.setAlignment(Label.CENTER);
    label2.setText("Nb couleurs^1/3");
    txtCouleurs.setEnabled(false);
    txtCouleurs.setText(Integer.toString(ReseauSOMColor.NB_COL_DEF));
    label3.setAlignment(Label.CENTER);
    label3.setLocale(java.util.Locale.getDefault());
    label3.setText("Itrations");
    txtIterations.setEnabled(false);
    txtIterations.setText(Integer.toString(ReseauSOMColor.MAX_ITER_DEF));
    panel3.setLayout(gridLayout3);
    gridLayout3.setColumns(11);
    label4.setAlignment(Label.CENTER);
    label4.setText("Eta0");
    txtEta0.setEnabled(false);
    txtEta0.setText(Double.toString(ReseauSOMColor.ETA0_DEF));
    label5.setAlignment(Label.CENTER);
    label5.setText("Sigma0");
    txtSigma0.setEnabled(false);
    txtSigma0.setText(Double.toString(ReseauSOMColor.SIGMA0_DEF));
    txtTaille.setEnabled(false);
    txtTaille.setText(Integer.toString(ReseauSOMColor.TAILLE_DEF));
    btnModif.setLabel("Modif.");
    btnModif.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnModif_actionPerformed(e);
      }
    });
    this.add(statusBar, BorderLayout.SOUTH);
    this.add(panel1, BorderLayout.CENTER);
    panel1.add(panel2,  BorderLayout.SOUTH);
    panel2.add(btnReset, null);
    panel2.add(btnGo, null);
    panel2.add(btnPas, null);
    panel2.add(btnAbout, null);
    panel1.add(panelVue, BorderLayout.CENTER);
    this.add(panel3, BorderLayout.NORTH);
    panel3.add(label1, null);
    panel3.add(txtTaille, null);
    panel3.add(label2, null);
    panel3.add(txtCouleurs, null);
    panel3.add(label3, null);
    panel3.add(txtIterations, null);
    panel3.add(label4, null);
    panel3.add(txtEta0, null);
    panel3.add(label5, null);
    panel3.add(txtSigma0, null);
    panel3.add(btnModif, null);
    // ajout de la vue
    panelVue.add(processSOM.somVue, null);
    initParam();
    processSOM.initReseau();
  }

  /** Rinitialisation */
  void btnReset_actionPerformed(ActionEvent e) {
    if (processSOM.isRunning == true) {
      btnGo.setLabel("GO");
      processSOM.stop();
      setInterface(true);
      processSOM.reInit();
      processSOM.lancement();
      btnGo.setLabel("Stop");
    }
    else {
      initParam();
      processSOM.reInit();
      affGeneration();
    }
  }

  /** Lance l'apprentissage */
  void btnGo_actionPerformed(ActionEvent e) {
    stopGo();
  }

  /** Stop/Go */
  public void stopGo() {
    if (processSOM.isRunning == false) {
      btnGo.setLabel("Stop");
      setInterface(false);
      processSOM.lancement();
    }
    else {
      btnGo.setLabel("GO");
      processSOM.stop();
      setInterface(true);
    }
  }

  /** Pas  pas */
  void btnPas_actionPerformed(ActionEvent e) {
    processSOM.leReseau.affRes();
  }

  void btnAbout_actionPerformed(ActionEvent e) {
    dispInfoBox();
  }

  /** (In)active l'interface */
  public void setInterface(boolean e) {
    btnReset.setEnabled(e);
    btnPas.setEnabled(e);
    btnAbout.setEnabled(e);
    btnModif.setEnabled(e);
  }

  /** Affichage de la gnration */
  public void affGeneration() {
    statusBar.setText(" Itration :"
                      + Integer.toString(processSOM.generation));
  }

  /** Redimensionnement */
  void panelVue_componentResized(ComponentEvent e) {
    redimUi();
  }

  /** Display Info box */
  void dispInfoBox() {
    String s =  applet.product + "\n"
        + "Version : " + applet.version + ", "
        + applet.date + "\n\n"
        + applet.comments + "\n"
        + applet.auteur + "\n"
        + applet.editeur +"\n"
        + applet.copyright + "\n\n"
        + applet.GNU1 + "\n" + applet.GNU2 + "\n" + applet.GNU3 + "\n";
    CaboutBox aboutBox = new CaboutBox(this, "Classification des couleurs", true);
    aboutBox.dispInfoBox(s);
  }

  /** Initialisation/MAJ paramtres rseau */
  public void initParam() {
    Calert x;
    String msg = "Certains paramtres n'taient pas corrects. Ils ont t remplacs par la valeur par dfaut.";
    boolean valide = true;
    boolean ttValide = true;
    int n = 0;
    double d = 0.0D;
    // Taille
    try {
      n = Integer.parseInt(txtTaille.getText());
    } catch(NumberFormatException er) {
      valide = false;
    }
    if(n <= 0 || n > 100) valide = false;
    if(!valide) {
      txtTaille.setText(Integer.toString(ReseauSOMColor.TAILLE_DEF));
      n = ReseauSOMColor.TAILLE_DEF;
      ttValide = false;
    }
    tailleReseau = n;
    // Itrations maximum
    valide = true;
    try {
      n = Integer.parseInt(txtIterations.getText());
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(n <= 0 || n > 100000) valide = false;
    if(!valide) {
      txtIterations.setText(Integer.toString(ReseauSOMColor.MAX_ITER_DEF));
      n = ReseauSOMColor.MAX_ITER_DEF;
      ttValide = false;
    }
    maxIterations = n;
    // Nb couleurs
    valide = true;
    try {
      n = Integer.parseInt(txtCouleurs.getText());
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(n <= 0 || n > 20) valide = false;
    if(!valide) {
      txtCouleurs.setText(Integer.toString(ReseauSOMColor.NB_COL_DEF));
      n = ReseauSOMColor.NB_COL_DEF;
      ttValide = false;
    }
    nbCouleurs = n;
    // Eta0
    valide = true;
    try {
      d = Double.valueOf(txtEta0.getText()).doubleValue();
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(d <= 0 || d > 1) valide = false;
    if(!valide) {
      txtEta0.setText(Double.toString(ReseauSOMColor.ETA0_DEF));
      d = ReseauSOMColor.ETA0_DEF;
      ttValide = false;
    }
    eta0 = d;
    // Sigma0
    valide = true;
    try {
      d = Double.valueOf(txtSigma0.getText()).doubleValue();
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(d <= 0 || d > 1) valide = false;
    if(!valide) {
      txtSigma0.setText(Double.toString(ReseauSOMColor.SIGMA0_DEF));
      d = ReseauSOMColor.SIGMA0_DEF;
      ttValide = false;
    }
    sigma0 = d;
    if(!ttValide) {
      x = new Calert(this, msg);
    }
  }

  /** Modification des paramtres rseau */
  void btnModif_actionPerformed(ActionEvent e) {
    if(btnModif.getLabel() == "Modif.") {
      btnModif.setLabel("Valide");
      txtTaille.setEnabled(true);
      txtIterations.setEnabled(true);
      txtCouleurs.setEnabled(true);
      txtEta0.setEnabled(true);
      txtSigma0.setEnabled(true);
      btnReset.setEnabled(false);
      btnPas.setEnabled(false);
      btnGo.setEnabled(false);
      btnAbout.setEnabled(false);
    }
    else {
      btnModif.setLabel("Modif.");
      txtTaille.setEnabled(false);
      txtIterations.setEnabled(false);
      txtCouleurs.setEnabled(false);
      txtEta0.setEnabled(false);
      txtSigma0.setEnabled(false);
      btnReset.setEnabled(true);
      btnPas.setEnabled(true);
      btnGo.setEnabled(true);
      btnAbout.setEnabled(true);
      initParam();
      processSOM.reInit();
      affGeneration();
    }
  }
}
